module PrePlaySports

  class Event
    attr_accessor :event_type, :data, :context

    def initialize(event_type, data, context)
      @event_type = event_type
      @data = filter(data)
      @context = context
    end

    def to_scrolls_hash
      event = {
        d: { type: event_type }.merge!(data),
        c: context,
        m: {
          dyno: $dyno_name,
          created_at: Time.now.to_s
        }
      }
      flatten_keys(event).merge!(preplay_event: true)
    end

    private

    #this "flatten" a hash in order to use it on a Scrolls.log line
    # {
    #   data: {
    #     foo: 'bar',
    #     baz: 42
    #   }
    # }
    # become :
    # {
    #   :'data.foo' => 'bar',
    #   :'data.baz' => 42
    # }
    def flatten_keys(hash, namespace = nil)
      namespace = namespace.nil? ? '' : "#{namespace}."
      hash.inject(Hash.new) do |result, key, value|

        hash.each do |k, v|
          if v.is_a? Hash
            # recursive call with the new namespace
            result.merge! flatten_keys(v, namespace + k.to_s)
          else
            # direct set in the hash of the v
            result["#{namespace}#{k}".to_sym] = v
          end
        end
        result
      end
    end

    #will filter the hash that was given to keep only the data that we want to display on the logs dependending on the data type
    FIELD_WHITELIST = {
      'guess_created' => %w{id choice},
      'guess_updated' => %w{id choice},
      'message_published' => %w{id channel_id content tags},
      'user_follow' => %w{id username},
      'user_sign_up' => %w{id},
      'user_image_url_updated' => %w{id after before},
      'user_username_updated' => %w{id after before},
      'user_muted' => %w{id muted_until},
      'user_blacklisted' => %w{id ip},
    }.freeze

    def filter(data)
      return {} unless FIELD_WHITELIST[event_type]
      data.select {|k| FIELD_WHITELIST[event_type].include? k.to_s}
    end
  end

  def self.Event(event_type, data, context)
    Event.new(event_type, data, context).to_scrolls_hash
  end

end
